
import 'package:equatable/equatable.dart';

abstract class SearchState extends Equatable {
  SearchState([List props = const []]) : super();

  @override
  List<Object> get props => [];
}

class GetDataSuccess extends SearchState {
  @override
  String toString() {
    return 'GetDataSuccess{}';
  }
}

class SearchSuccess extends SearchState {
  @override
  String toString() {
    return 'SearchSuccess{}';
  }
}

class SearchInitial extends SearchState {
  @override
  String toString() {
    return 'SearchInitial{}';
  }
}

class SearchLoading extends SearchState {
  @override
  String toString() {
    return 'SearchLoading{}';
  }
}

class SearchFailure extends SearchState {
  final String error;

  SearchFailure(this.error);

  @override
  String toString() {
    return 'SearchFailure {error: $error}';
  }
}